==> fmenu-0.1.tar.gz: fmenu-0.1/main.c <==
/*
 * (C)opyright MMVI Anselm R. Garbe <garbeam at gmail dot com>
 * (C)opyright MMVI Sander van Dijk <a dot h dot vandijk at gmail dot com>
 * (C)opyright MMVI Jesus Galan (yiyus) <yiyu.jgl at gmail dot com>
 * See LICENSE file for license details.
 */

#include "fmenu.h"

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/time.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

typedef struct Item Item;
struct Item {
	Item *next;		/* traverses all items */
	Item *left, *right;	/* traverses items matching current search pattern */
	char *text; // <-- TODO: remove
	char key;
	char *out;
	char *info;
};

/* static */

static char text[4096];
static int mx, my, mw, mh;
static int ret = 0;
static int nitem = 0;
static unsigned int cmdw = 0;
static Bool running = True;
static Item *allitems = NULL;	/* first of all items */
static Item *item = NULL;	/* first of pattern matching items */
static Item *sel = NULL;
static Item *next = NULL;
static Item *prev = NULL;
static Item *curr = NULL;
static Window root;
static Window win;

static void
drawmenu(void) {
	Item *i;

	dc.x = 0;
	dc.y = 0;
	dc.w = mw;
	dc.h =  dc.font.height + 2;;
	drawtext(NULL, dc.norm);

	/* print command */
	//if(cmdw && item)
	//	dc.w = cmdw;
	//drawtext(text[0] ? text : NULL, dc.norm);

	if(item) {
		//dc.w = SPACE;
		drawtext(NULL, dc.norm);
		dc.y += dc.h;
		//dc.w = mw;

		/* determine maximum items */
		for(i = item; i != next; i=i->next) {
			//dc.w = textw(i->text);
			//if(dc.w > mw / 3)
			//	dc.w = mw / 3;

			if(i->info) {
				drawtext(i->info, dc.norm);
				dc.y += dc.h;
			}
		}

		//dc.x = mw - SPACE;
		//dc.w = SPACE;
		drawtext(NULL, dc.norm);
	}
	XCopyArea(dpy, dc.drawable, win, dc.gc, 0, 0, mw, mh, 0, 0);
	XFlush(dpy);
}

static void
kpress(XKeyEvent * e) {
	char buf[32];
	int num, prev_nitem;
	unsigned int i, len;
	Item *it;
	KeySym ksym;

	len = strlen(text);
	buf[0] = 0;
	num = XLookupString(e, buf, sizeof(buf), &ksym, 0);

	if(IsFunctionKey(ksym) || IsKeypadKey(ksym)
			|| IsMiscFunctionKey(ksym) || IsPFKey(ksym)
			|| IsPrivateKeypadKey(ksym))
		return;
	for(it = item; it != next; it=it->next) {
		if(it->key == buf[0]) {
			fprintf(stdout, "%s", it->out);
			exit(0);
		}
	}

	exit(0);
}

static char *
readstdin(void) {
	static char *maxname = NULL;
	char *p, buf[1024], key;
	unsigned int len = 0, max = 0;
	Item *i, *new;

	i = 0;
	while(fgets(buf, sizeof(buf), stdin)) {
		len = strlen(buf);
		if (buf[len - 1] == '\n')
			buf[len - 1] = 0;
		p = estrdup(buf);

		if(max < len) {
			maxname = p;
			max = len;
		}

		new = emalloc(sizeof(Item));
		new->next = NULL;
		new->info = NULL;
		new->text = p;

		if(!sscanf(p, "!%c ", &key)){
			sscanf(p, "%c ", &key);
			new->info = p;
			new->out = (p + 2);
			nitem++;
		} else
			new->out = (p + 3);
		new->key = key;
		
		if(!i)
			item = new;
		else 
			i->next = new;
		i = new;
	}

	return maxname;
}

/* extern */

int screen;
Display *dpy;
DC dc = {0};

int
main(int argc, char *argv[]) {
	char *font = FONT;
	char *maxname;
	char *normbg = NORMBGCOLOR;
	char *normfg = NORMFGCOLOR;
//	char *selbg = SELBGCOLOR;
//	char *selfg = SELFGCOLOR;
	fd_set rd;
	int i;
	struct timeval timeout;
	Item *itm;
	XEvent ev;
	XSetWindowAttributes wa;

	timeout.tv_usec = 0;
	timeout.tv_sec = 3;
	/* command line args */
	for(i = 1; i < argc; i++)
		if(!strncmp(argv[i], "-font", 6))
			font = argv[++i];
		else if(!strncmp(argv[i], "-normbg", 8))
			normbg = argv[++i];
		else if(!strncmp(argv[i], "-normfg", 8))
			normfg = argv[++i];
		else if(!strncmp(argv[i], "-t", 3))
			timeout.tv_sec = atoi(argv[++i]);
		else if(!strncmp(argv[i], "-v", 3)) {
			fputs("fmenu-"VERSION", (C)opyright MMVI Jesus Galan and Anselm R. Garbe\n", stdout);
			exit(EXIT_SUCCESS);
		}
		else
			eprint("usage: fmenu [-font <name>] [-norm{bg,fg} <color>] [-t <seconds>] [-v]\n", stdout);

	dpy = XOpenDisplay(0);
	if(!dpy)
		eprint("fmenu: cannot open display\n");
	screen = DefaultScreen(dpy);
	root = RootWindow(dpy, screen);

	/* Note, the select() construction allows to grab all keypresses as
	 * early as possible, to not loose them. But if there is no standard
	 * input supplied, we will make sure to exit after MAX_WAIT_STDIN
	 * seconds. This is convenience behavior for rapid typers.
	 */ 
	while(XGrabKeyboard(dpy, root, True, GrabModeAsync,
			 GrabModeAsync, CurrentTime) != GrabSuccess)
		usleep(1000);

	FD_ZERO(&rd);
	FD_SET(STDIN_FILENO, &rd);
	if(select(ConnectionNumber(dpy) + 1, &rd, NULL, NULL, &timeout) < 1)
		goto UninitializedEnd;
	nitem = 0;
	maxname = readstdin();

	/* style */
	dc.norm[ColBG] = getcolor(normbg);
	dc.norm[ColFG] = getcolor(normfg);
	setfont(font);

	wa.override_redirect = 1;
	wa.background_pixmap = ParentRelative;
	wa.event_mask = ExposureMask | ButtonPressMask | KeyPressMask;

	mw = textw(maxname);
	mh = (nitem + 2) * (dc.font.height + 2);
	mx = (DisplayWidth(dpy, screen) - mw) / 2;
	my = (DisplayHeight(dpy, screen) - mh) /2;
	
	win = XCreateWindow(dpy, root, mx, my, mw, mh, 0,
			DefaultDepth(dpy, screen), CopyFromParent,
			DefaultVisual(dpy, screen),
			CWOverrideRedirect | CWBackPixmap | CWEventMask, &wa);
	XDefineCursor(dpy, win, XCreateFontCursor(dpy, XC_xterm));

	/* pixmap */
	dc.drawable = XCreatePixmap(dpy, root, mw, mh, DefaultDepth(dpy, screen));
	dc.gc = XCreateGC(dpy, root, 0, 0);
	XSetLineAttributes(dpy, dc.gc, 1, LineSolid, CapButt, JoinMiter);

	if(maxname)
		cmdw = textw(maxname);

	XMapRaised(dpy, win);
	drawmenu();
	XSync(dpy, False);

	/* main event loop */
	while(running && !XNextEvent(dpy, &ev)) {
		switch (ev.type) {
		default:	/* ignore all crap */
			break;
		case KeyPress:
			kpress(&ev.xkey);
			break;
		case Expose:
			if(ev.xexpose.count == 0)
				drawmenu();
			break;
		}
	}

/*	while(allitems) {
		itm = allitems->next;
		free(allitems->text);
		free(allitems);
		allitems = itm;
	}
*/
	if(dc.font.set)
		XFreeFontSet(dpy, dc.font.set);
	else
		XFreeFont(dpy, dc.font.xfont);
	XFreePixmap(dpy, dc.drawable);
	XFreeGC(dpy, dc.gc);
	XDestroyWindow(dpy, win);
UninitializedEnd:
	XUngrabKeyboard(dpy, CurrentTime);
	XCloseDisplay(dpy);

	return ret;
}

==> fmenu-0.2.tar.gz: fmenu-0.2/main.c <==
/*
 * (C)opyright MMVI Anselm R. Garbe <garbeam at gmail dot com>
 * (C)opyright MMVI Sander van Dijk <a dot h dot vandijk at gmail dot com>
 * (C)opyright MMVI Jesus Galan (yiyus) <yiyu.jgl at gmail dot com>
 * See LICENSE file for license details.
 */

#include "fmenu.h"

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/time.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

typedef struct Item Item;
struct Item {
	Item *next;		/* traverses all items */
	char key;
	char *out;
	char *info;
};

/* static */

static int mx, my, mw, mh;
static int ret = 0;
static int nitem = 0;
static Bool running = True;
static Item *item = NULL;	/* first of pattern matching items */
static Item *next = NULL;
static Window root;
static Window win;

static void
drawmenu(void) {
	Item *i;

	dc.x = 0;
	dc.y = 0;
	dc.w = mw;
	dc.h =  dc.font.height + 2;;
	drawtext(NULL, dc.norm);

	if(item) {
		drawtext(NULL, dc.norm);
		dc.y += dc.h;

		/* determine maximum items */
		for(i = item; i != next; i=i->next) {
			if(i->info) {
				drawtext(i->info, dc.norm);
				dc.y += dc.h;
			}
		}

		drawtext(NULL, dc.norm);
	}
	XCopyArea(dpy, dc.drawable, win, dc.gc, 0, 0, mw, mh, 0, 0);
	XFlush(dpy);
}

static void
kpress(XKeyEvent * e) {
	char buf[32];
	int num, prev_nitem;
	Item *i;
	KeySym ksym;

	buf[0] = 0;
	num = XLookupString(e, buf, sizeof(buf), &ksym, 0);

	if(IsFunctionKey(ksym) || IsKeypadKey(ksym)
			|| IsMiscFunctionKey(ksym) || IsPFKey(ksym)
			|| IsPrivateKeypadKey(ksym))
		return;
	for(i = item; i != next; i=i->next) {
		if(i->key == buf[0]) {
			fprintf(stdout, "%s", i->out);
			exit(0);
		}
	}

	exit(0);
}

static char *
readstdin(void) {
	static char *maxname = NULL;
	char *p, buf[1024], key;
	unsigned int len = 0, max = 0;
	Item *i, *new;

	i = 0;
	while(fgets(buf, sizeof(buf), stdin)) {
		len = strlen(buf);
		if (buf[len - 1] == '\n')
			buf[len - 1] = 0;
		p = estrdup(buf);

		new = emalloc(sizeof(Item));
		new->next = NULL;
		new->info = NULL;

		if(!sscanf(p, "!%c ", &key)){
			sscanf(p, "%c ", &key);
			new->info = p;
			new->out = (p + 2);

			if(max < len) {
				maxname = p;
				max = len;
			}

			nitem++;
		} else
			new->out = (p + 3);
		new->key = key;
		
		if(!i)
			item = new;
		else 
			i->next = new;
		i = new;
	}

	return maxname;
}

/* extern */

int screen;
Display *dpy;
DC dc = {0};

int
main(int argc, char *argv[]) {
	char *font = FONT;
	char *maxname;
	char *normbg = NORMBGCOLOR;
	char *normfg = NORMFGCOLOR;
	fd_set rd;
	int i;
	struct timeval timeout;
	Item *itm;
	XEvent ev;
	XSetWindowAttributes wa;

	timeout.tv_usec = 0;
	timeout.tv_sec = 3;
	/* command line args */
	for(i = 1; i < argc; i++)
		if(!strncmp(argv[i], "-font", 6))
			font = argv[++i];
		else if(!strncmp(argv[i], "-normbg", 8))
			normbg = argv[++i];
		else if(!strncmp(argv[i], "-normfg", 8))
			normfg = argv[++i];
		else if(!strncmp(argv[i], "-t", 3))
			timeout.tv_sec = atoi(argv[++i]);
		else if(!strncmp(argv[i], "-v", 3)) {
			fputs("fmenu-"VERSION", (C)opyright MMVI Jesus Galan and Anselm R. Garbe\n", stdout);
			exit(EXIT_SUCCESS);
		}
		else
			eprint("usage: fmenu [-font <name>] [-norm{bg,fg} <color>] [-t <seconds>] [-v]\n", stdout);

	dpy = XOpenDisplay(0);
	if(!dpy)
		eprint("fmenu: cannot open display\n");
	screen = DefaultScreen(dpy);
	root = RootWindow(dpy, screen);

	/* Note, the select() construction allows to grab all keypresses as
	 * early as possible, to not loose them. But if there is no standard
	 * input supplied, we will make sure to exit after MAX_WAIT_STDIN
	 * seconds. This is convenience behavior for rapid typers.
	 */ 
	while(XGrabKeyboard(dpy, root, True, GrabModeAsync,
			 GrabModeAsync, CurrentTime) != GrabSuccess)
		usleep(1000);

	FD_ZERO(&rd);
	FD_SET(STDIN_FILENO, &rd);
	if(select(ConnectionNumber(dpy) + 1, &rd, NULL, NULL, &timeout) < 1)
		goto UninitializedEnd;
	nitem = 0;
	maxname = readstdin();

	/* style */
	dc.norm[ColBG] = getcolor(normbg);
	dc.norm[ColFG] = getcolor(normfg);
	setfont(font);

	wa.override_redirect = 1;
	wa.background_pixmap = ParentRelative;
	wa.event_mask = ExposureMask | ButtonPressMask | KeyPressMask;

	mw = textw(maxname);
	mh = (nitem + 2) * (dc.font.height + 2);
	mx = (DisplayWidth(dpy, screen) - mw) / 2;
	my = (DisplayHeight(dpy, screen) - mh) /2;
	
	win = XCreateWindow(dpy, root, mx, my, mw, mh, 0,
			DefaultDepth(dpy, screen), CopyFromParent,
			DefaultVisual(dpy, screen),
			CWOverrideRedirect | CWBackPixmap | CWEventMask, &wa);
	XDefineCursor(dpy, win, XCreateFontCursor(dpy, XC_xterm));

	/* pixmap */
	dc.drawable = XCreatePixmap(dpy, root, mw, mh, DefaultDepth(dpy, screen));
	dc.gc = XCreateGC(dpy, root, 0, 0);
	XSetLineAttributes(dpy, dc.gc, 1, LineSolid, CapButt, JoinMiter);

	XMapRaised(dpy, win);
	drawmenu();
	XSync(dpy, False);

	/* main event loop */
	while(running && !XNextEvent(dpy, &ev)) {
		switch (ev.type) {
		default:	/* ignore all crap */
			break;
		case KeyPress:
			kpress(&ev.xkey);
			break;
		case Expose:
			if(ev.xexpose.count == 0)
				drawmenu();
			break;
		}
	}

	if(dc.font.set)
		XFreeFontSet(dpy, dc.font.set);
	else
		XFreeFont(dpy, dc.font.xfont);
	XFreePixmap(dpy, dc.drawable);
	XFreeGC(dpy, dc.gc);
	XDestroyWindow(dpy, win);
UninitializedEnd:
	XUngrabKeyboard(dpy, CurrentTime);
	XCloseDisplay(dpy);

	return ret;
}
